{$A+,B-,C-,D-,E-,F-,G+,H+,I-,J-,K-,L-,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y-,Z1}

{*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * Author:       Jerry Claxton                                               *
 * Description:  Displays Disk/File Usage in Selected Folders.               *
 * Creation:     May 18th, 2002                                              *
 * Version:      1.00                                                        *
 * EMail:        gclaxton@ameritech.net                                      *
 *               jerryclaxton@digitalme.com                                  *
 *                                                                           *
 * Support:      Use the email gclaxton@ameritech.net                        *
 * Legal issues: Copyright (C) 1996-2002 by Jerry Claxton                    *
 *               210 Village Creek Dr, Lake in the Hills, Illinois, 60156    *
 *               <gclaxton@ameritech.net> <jerryclaxton@digitalme.com>       *
 *                                                                           *
 *               This software is provided 'as-is', without any express or   *
 *               implied warranty.  In no event will the author be held      *
 *               liable for any  damages arising from the use of this        *
 *               software.                                                   *
 *                                                                           *
 *               Permission is granted to anyone to use this software for    *
 *               any purpose, including commercial applications, and to      *
 *               alter it and redistribute it freely, subject to the         *
 *               following restrictions:                                     *
 *                                                                           *
 *               1. The origin of this software must not be misrepresented,  *
 *                  you must not claim that you wrote the original software. *
 *                  If you use this software in a product, an acknowledgment *
 *                  in the product documentation would be appreciated but is *
 *                  not required.                                            *
 *                                                                           *
 *               2. Altered source versions must be plainly marked as such,  *
 *                  and must not be misrepresented as being the original     *
 *                  software.                                                *
 *                                                                           *
 *               3. This notice may not be removed or altered from any       *
 *                  source distribution.                                     *
 *                                                                           *
 *               4. You must register this software by sending an email to   *
 *                  the author. Any comment you like to say.                 *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * History:                                                                  *
 * May 20, 2002  First Release.                                              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *}

unit ScanThrd;

interface

uses
  Windows, Classes, SysUtils;

type
  TProgressEvent = procedure(Folder: string) of object;
  TProccessEvent = procedure(Folder: string; TotalSize: Int64) of object;

  TMyThread = class(TThread)
  private
    TotalSize: Int64;
    Folder: string;
    fOnProgress: TProgressEvent;
    fOnProccess: TProccessEvent;
    procedure ShowProgress;
    procedure ShowProccess;
  protected
    fTotalSize: Int64;
    fPath: string;
    procedure DoProgress(Folder: string); virtual;
    procedure DoProccess(Folder: string; TotalSize: Int64); virtual;
  public
  published
    constructor Create(const sPath: string);
    destructor Destroy; override;
    property OnProgress: TProgressEvent read FOnProgress write FOnProgress;
    property OnProccess: TProccessEvent read fOnProccess write fOnProccess;
  end;

  TScanThread = class(TMyThread)
  private
    procedure DirSize(const sPath: string);
    procedure GetDirs(const sPath: string);
  protected
    procedure Execute; override;
  public
  published
    constructor Create(const sPath: string);
  end;

implementation

{TMyThread}
constructor TMyThread.Create(const sPath: string);
begin
  inherited Create(True);
  FreeOnTerminate := True;

  fPath := sPath;
  fTotalSize := 0;
end;

destructor TMyThread.Destroy;
begin
  inherited Destroy;
end;

procedure TMyThread.DoProccess(Folder: string; TotalSize: Int64);
begin
  if Assigned(fOnProccess) then fOnProccess(Folder, TotalSize);
end;

procedure TMyThread.ShowProccess;
begin
  DoProccess(Folder, TotalSize);
end;

procedure TMyThread.DoProgress(Folder: string);
begin
  if Assigned(fOnProgress) then fOnProgress(Folder);
end;

procedure TMyThread.ShowProgress;
begin
  DoProgress(Folder);
end;

{TScanThread}
constructor TScanThread.Create(const sPath: string);
begin
  inherited Create(sPath);
end;

procedure TScanThread.DirSize(const sPath: string);
var
  SR: TSearchRec;
begin
  FindFirst(sPath + '*.*', faAnyFile, SR);
  try
    repeat
      if ((SR.Attr and faDirectory) = faDirectory) and
        (SR.Name <> '.') and (SR.Name <> '..') then
        DirSize(sPath + SR.Name + '\')
      else
        if (SR.Name <> '.') and (SR.Name <> '..') then
          fTotalSize := fTotalSize + SR.Size;
    until (FindNext(SR) <> 0) or (Terminated);
  finally
    FindClose(SR);
  end;
end;

procedure TScanThread.GetDirs(const sPath: string);
var
  SR: TSearchRec;
begin
  FindFirst(sPath + '*.*', faAnyFile, SR);
  try
    repeat
      if ((SR.Attr and faDirectory) = faDirectory) and
        (SR.Name <> '.') and (SR.Name <> '..') then
      begin
        Folder := sPath + SR.Name;
        Synchronize(ShowProgress);

        DirSize(sPath + SR.Name + '\');

        TotalSize := fTotalSize;
        Synchronize(ShowProccess);
      end;
      fTotalSize := 0;
    until (FindNext(SR) <> 0) or (Terminated);
  finally
    FindClose(SR);
  end;
end;

procedure TScanThread.Execute;
begin
  GetDirs(fPath);
end;

end.
